public with sharing class TestSessionFields {

  public void testSessionFields(){
  		//prepare for session fields
  		Map<String,String> sessionFields = new Map<String,String>();
  		sessionFields.put('username','123');
  		sessionFields.put('password','456');
  	
		String signerAuthenticationToken = createSignerAuthenticationToken('T2NYsTrrrrccJqwABfGLTGo9mNk=','esignguide2@mailinator.com',sessionFields);
		String redirctUrl = buildRedirectToSigningForSigner(signerAuthenticationToken);
		System.debug('redirct Url: '+ redirctUrl);
	}
	
	public String buildRedirectToSigningForSigner(String authenticationToken){
		e_SignLive_Connection_Settings__c connectionSettings = e_SignLive_Connection_Settings__c.getInstance('Main');    
        String endpointWithoutAPI = connectionSettings.Endpoint__c.substringBeforeLast('/api');
        return endpointWithoutAPI + '/access?sessionToken='+ authenticationToken;
	}
	
    public String createSignerAuthenticationToken(String packageId, String signerId, Map<String,String> sessionFields){
    	String requestPayload = '{  "packageId": "'+packageId+'",  "signerId": "'+signerId+'"';
    	
    	if(sessionFields != null && sessionFields.size() > 0){
    		requestPayload += ',"sessionFields": '+ JSON.serialize(sessionFields);
    	}
    	
    	requestPayload += '}';
    	String resourceUrl = '/signerAuthenticationTokens';
        HttpResponse response = doPost(resourceUrl, requestPayload);
		
        // Check response
        if(response.getStatusCode() == 200){
        	try
            {
            	Map<String,Object> responseObject = (Map<String,Object>)JSON.deserializeUntyped(response.getBody());
            	String authenticationToken =  (String)responseObject.get('value');
            	return authenticationToken;
            }
            catch(Exception e)
            {
                throw new ESignLiveRESTAPIHelper.ESignLiveRestAPIHelperException('Error converting signer Authentication Tokens json ' + e.getMessage());
            }
        }else{
            throw new ESignLiveRESTAPIHelper.ESignLiveRestAPIHelperException('Error build Redirect To Signing For Signer ' + response.getStatusCode() + ' - ' + response.getStatus() + ' - ' +  response.getBody());
        }
    }
    
    private HttpResponse doPost(String resourceUrl, String requestBody)
    {
        e_SignLive_Connection_Settings__c connectionSettings = e_SignLive_Connection_Settings__c.getInstance('Main');    
        //Cleanup requestBody string before sending it out
        String outboundRequestBody = ESignLiveJSONHelper.prepareOutboundJSON(requestBody);
        System.debug('=> outboundRequestBody:'+outboundRequestBody);

        // Build Request
        HttpRequest request = new HttpRequest();

        request.setEndpoint(connectionSettings.Endpoint__c + resourceUrl);
        request.setBody(outboundRequestBody);
        request.setMethod('POST');
        request.setHeader('Authorization', 'Basic ' + connectionSettings.API_Key__c);
        request.setHeader('Content-type', 'application/json');
        request.setHeader('Accept', 'application/json');

        // Send Request
        HttpResponse response = (new Http()).send(request);

        return response;
    }
  
  
    public String createPackage() {
        //Create esl client
        ESignLiveSDK sdk = new ESignLiveSDK();
         
        //Create package
        ESignLiveAPIObjects.Package_x pkg = new ESignLiveAPIObjects.Package_x();
        pkg.name = 'Test Session Fields - ' + Datetime.now().format();
         
        String packageId = sdk.createPackage(pkg);
        System.debug('PackageId: ' + packageId);
        
        //Add Signer
        String roleResponse = sdk.addRole('John', 'Smith', 'signer1@mailinator.com', packageId);
         
        //Add Document
        StaticResource sr = [SELECT Id, Body FROM StaticResource WHERE Name = 'testdoc1' LIMIT 1];
        Map<String,Blob> doc = new Map<String,Blob>();
        doc.put('Sample Document', sr.Body);
         
        ESignLiveAPIObjects.Document document = new ESignLiveAPIObjects.Document();
        document.name = 'Sample Contract';
        document.id = 'document1';
         
        sdk.createDocuments(packageId, document, doc);
        

        //Add Signature
        Map<String,Object> role = (Map<String,Object>)JSON.deserializeUntyped(roleResponse);
        String roleId = (String)(role.get('id'));
         
        ESignLiveAPIObjects.Field field = new ESignLiveAPIObjects.Field();
        field.left = 208;
        field.width = 200;
        field.height = 50;
        field.top = 518;
        field.page = 0;
        field.subtype = 'FULLNAME';
        field.type = 'SIGNATURE';
         
        List<ESignLiveAPIObjects.Field> fields = new List<ESignLiveAPIObjects.Field>();
        fields.add(field);
         
        ESignLiveAPIObjects.Approval approval = new ESignLiveAPIObjects.Approval();
        approval.fields = fields;
        approval.role = roleId;
         
        String signatureResponse = sdk.addSignature(packageId, 'document1', approval);
         
        //Send package
        pkg.status = ESignLiveAPIObjects.PackageStatus.SENT;
         
        sdk.updatePackage(pkg, packageId);  
        
        return packageId;   
    }
}